unit cDIBControl;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: cDIBControl.PAS, released August 28, 2000.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2000 Peter Morris.
All Rights Reserved.

Purpose of file:
The main base component for all DIB components

Contributor(s):
None as yet


Last Modified: September 01, 2000
Current Version: 1.00

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org
or at http://www.stuckindoors.com/dib


Known Issues:
Would have been good if we could have derived this component from TCustomControl
instead.  If anyone can work out how to get this stuff to work just as well from
a TCustomControl I would love to hear from you !!!
-----------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, cDIB,
  cDIBPanel, cDIBFeatures, ExtCtrls, Consts, Menus, cDIBTimer, cDIBImageList, TypInfo;

type
  TWantedKey = (wkTab, wkArrows, wkAll);
  TWantedKeys = Set of TWantedKey;

  EDIBControlError = class(Exception);
  TCustomDIBControl = class(TControl)
  private
    { Private declarations }
    FPropertyList   : TList;
    FDIBImageList   : TDIBImageList;
    FCanvas         : TCanvas;
    FControlDIB     : TWinDIB;
    FAccelerator    : Char;
    FCreating       : Boolean;
    FFocused        : Boolean;
    FStoppingRepeat : Boolean;
    FAlreadyMoving  : Boolean;
    FChildren       : TControlList;
    FMouseRepeat    : Boolean;
    FMouseRepeatInterval : Integer;
    FTimer          : TDIBTimer;
    FLastMouse      : TMessage;
    FRealMouseInControl,
    FMouseInControl : Boolean;
    FOnEnter,
    FOnExit,
    FOnMouseEnter,
    FOnMouseLeave   : TNotifyEvent;
    FDIBFeatures    : TDIBFeatures;
    FOpacity        : Byte;
    FOnKeyDown      : TKeyEvent;
    FOnKeyUp        : TKeyEvent;
    FOnKeyPress     : TKeyPressEvent;

    //For streaming of properties to templates
    FPropertyNames  : TStringList;
    procedure ReadProperties(S : TStream);
    procedure WriteProperties(S : TStream);

    procedure DoImageChanged(Sender : TObject; Index : Integer; Operation : TDIBOperation);
    function  GetContainer : TCustomDIBContainer;
    function  GetTabOrder : TTabOrder;
    procedure SetOpacity(const Value: Byte);
    procedure SetTabOrder(const Value: TTabOrder);
    procedure RepeatMessage(Sender : TObject);

    procedure CMDialogChar(Var Message : TCMDialogChar); message CM_DialogChar;
    procedure WMChar(Var Message : TWMKey); message WM_Char;
    procedure WMGetDlgCode(Var Message : TWMGetDlgCode); message WM_GetDlgCode;
    procedure WMKeyDown(Var Message : TWMKey); message WM_KeyDown;
    procedure WMKeyUp(Var Message : TWMKey); message WM_KeyUp;
    procedure WMKillFocus(var Message : TMessage); message WM_KillFocus;
    procedure WMPAINT(Var Message : TMessage); message WM_Paint;
    procedure WMSetFocus(Var Message : TMessage); message WM_SetFocus;
    procedure SetDIBImageList(const Value: TDIBImageList);
  protected
    { Protected declarations }
    FTabOrder                 : TTabOrder;
    WantedKeys                : TWantedKeys;

    procedure AddTemplateProperty(const Name : String);
    procedure AlterUpdateRect(var R : TRect); virtual;
    procedure AddIndexProperty(var Index : TDIBImageLink);
    procedure Click; override;
    procedure DoAnyEnter; virtual;
    procedure DoAnyLeave; virtual;
    procedure DoEnter; virtual;
    procedure DoExit; virtual;
    function  DoKeyDown(var Message: TWMKey): Boolean; virtual;
    function  DoKeyPress(var Message: TWMKey): Boolean; virtual;
    function  DoKeyUp(var Message: TWMKey): Boolean; virtual;
    procedure DoMouseEnter; virtual;
    procedure DoMouseLeave; virtual;
    procedure KeyDown(var Key: Word; Shift: TShiftState); dynamic;
    procedure KeyUp(var Key: Word; Shift: TShiftState); dynamic;
    procedure KeyPress(var Key: Char); dynamic;
    procedure ImageChanged(Index : Integer; Operation : TDIBOperation); virtual;
    function  IsMenuKey(var Message: TWMKey): Boolean; virtual;
    function  IsMouseRepeating : Boolean;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; virtual;
    procedure SetParent(AParent : TWinControl); override;
    procedure StopRepeating; virtual;
    procedure WndProc(var Message : TMessage); override;

    property Accelerator      : Char
      read FAccelerator
      write FAccelerator;
    property Canvas           : TCanvas
      read FCanvas;
    property Children         : TControlList
      read FChildren
      write FChildren;
    property ControlDIB       : TWinDIB
      read FControlDIB
      write FControlDIB;
    property Creating         : Boolean
      read FCreating;
    property DIBFeatures      : TDIBFeatures
      read FDIBFeatures
      write FDIBFeatures;
    property DIBImageList     : TDIBImageList
      read FDIBImageList
      write SetDIBImageList;
    property LastMouse        : TMessage
      read FLastMouse;
    property MouseRepeat      : Boolean
      read FMouseRepeat
      write FMouseRepeat;
    property MouseRepeatInterval : integer
      read FMouseRepeatInterval
      write FMouseRepeatInterval;
    property Opacity          : Byte
      read FOpacity
      write SetOpacity;
    property DIBTabOrder      : TTabOrder
      read GetTabOrder
      write SetTabOrder;

    property OnEnter          : TNotifyEvent
      read FOnEnter
      write FOnEnter;
    property OnExit           : TNotifyEvent
      read FOnExit
      write FOnExit;
    property OnKeyDown        : TKeyEvent
      read FOnKeyDown
      write FOnKeyDown;
    property OnKeyPress       : TKeyPressEvent
      read FOnKeyPress
      write FOnKeyPress;
    property OnKeyUp          : TKeyEvent
      read FOnKeyUp
      write FOnKeyUp;
    property OnMouseEnter     : TNotifyEvent
      read FOnMouseEnter
      write FOnMouseEnter;
    property OnMouseLeave     : TNotifyEvent
      read FOnMouseLeave
      write FOnMouseLeave;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    procedure LoadTemplateFromFile(const Filename : TFilename);
    procedure LoadTemplateFromStream(const S : TStream); virtual;
    procedure SaveTemplateToFile(const Filename : TFilename);
    procedure SaveTemplateToStream(const S : TStream); virtual;

    procedure SetBounds(aLeft, aTop, aWidth, aHeight : Integer); override;
    procedure SetFocus;

    property Container        : TCustomDIBContainer
      read GetContainer;
    property Focused          : Boolean
      read FFocused;
    property MouseInControl   : Boolean
      read FMouseInControl;

  published
    { Published declarations }
  end;

//procedure AddDIBChildMessage(const Message : DWord);
//function  FindDIBChildMessage(const Message : DWord) : Boolean;
//procedure RemoveDIBChildMessage(const Message : DWord);

function GetPropertyName(aInstance : TComponent; aObject : TPersistent) : String;

implementation
uses
  ActiveX, COMObj;

type
  THackContainer = class(TCustomDIBContainer);
  THackFeatures = class(TDIBFeatures);
  THackControl = class(TControl);
  THackAbstractSuperDIB = class(TAbstractSuperDIB);
  THackWinControl = class(TWinControl);

  TDIBWrapper = class(TComponent)
  private
    FDIB : TMemoryDIB;
  published
    property DIB              : TMemoryDIB
      read FDIB
      write FDIB;
  end;

//var
//  FChildMessages              : TList;
//  CritMessageSection          : TCriticalSection;

{ Misc stuff }

(*
procedure AddDIBChildMessage(const Message : DWord);
begin
  CritMessageSection.Acquire;
  try
    if FChildMessages.IndexOf(Pointer(Message))= -1 then
      FChildMessages.Add(Pointer(Message));
  finally
    CritMessageSection.Leave;
  end;
end;

function  FindDIBChildMessage(const Message : DWord) : Boolean;
begin
  Result := FChildMessages.IndexOf(Pointer(Message)) > -1;
end;

procedure RemoveDIBChildMessage(const Message : DWord);
var
  Index                       : Integer;
begin
  CritMessageSection.Acquire;
  try
    Index := FChildMessages.IndexOf(Pointer(Message));
    if Index > -1 then FChildMessages.Delete(Index);
  finally
    CritMessageSection.Leave;
  end;
end;

*)


function GetPropertyName(aInstance : TComponent; aObject : TPersistent) : String;
var
  TI : PTypeInfo;
  PI : PPropInfo;
  Cnt : Integer;
  Props : PPropList;
begin
  Result := '';

  TI := aInstance.ClassInfo;
  Cnt := GetTypeData(TI)^.PropCount;
  GetMem(Props, Cnt * SizeOf(PPropInfo));
  Cnt := GetPropList(TI, [tkClass], Props);
  for Cnt:=Cnt-1 downto 0 do begin
    PI := GetPropInfo(TI, Props[Cnt].Name);
    if GetOrdProp(aInstance,PI) = LongInt(aObject) then begin
      Result := Props[Cnt].Name;
      break;
    end;
  end;
  FreeMem(Props);
end;



{ TCustomDIBControl }

constructor TCustomDIBControl.Create(AOwner: TComponent);
var
  X                 : Integer;
  P                 : TWinControl;
begin
  P := Nil;
  if AOwner = nil then Raise Exception.Create('Owner cannot be nil');

  if AOwner is TCustomDIBContainer then
    P := TWinControl(AOwner)
  else
    for X:=0 to AOwner.ComponentCount-1 do
      if AOwner.Components[X] is TCustomDIBContainer then begin
        P := TWinControl(AOwner.Components[X]);
        Break;
      end;

  if P = nil then Raise Exception.Create('Parent must be a TDIBContainer');
  inherited;
  FPropertyList := TList.Create;
  FPropertyNames := TStringList.Create;
  FCreating := True;
  FCanvas := TCanvas.Create;
  FAlreadyMoving := False;
  FStoppingRepeat := False;
  FChildren := TControlList.Create(Self);
  SetBounds(0,0,32,32);
  FDIBFeatures := TDIBFeatures.Create(Self);
  FOpacity := 255;
  FMouseInControl := False;
  FRealMouseInControl := False;
  FTimer := TDIBTimer.Create(nil);
  FTimer.OnTimer := RepeatMessage;
  FTimer.Enabled := False;
  FMouseRepeat := False;
  FMouseRepeatInterval := 250;
  FTabOrder := -1;
  WantedKeys := [wkArrows];
  AddTemplateProperty('Width');
  AddTemplateProperty('Height');
end;

destructor TCustomDIBControl.Destroy;
begin
  FPropertyNames.Free;
  FTimer.Free;
  FChildren.Free;
  FDIBFeatures.Free;
  FCanvas.Free;
  FPropertyList.Free;
  inherited;
end;

procedure TCustomDIBControl.Loaded;
begin
  inherited;
  THackFeatures(FDIBFeatures).Loaded;
  FCreating := False;
end;

function TCustomDIBControl.IsMenuKey(var Message: TWMKey): Boolean;
var
  Control: TControl;
  Form: TCustomForm;
begin
  Result := True;
  if not (csDesigning in ComponentState) then
  begin
    Control := Self;
    while Control <> nil do
    begin
      if Assigned(PopupMenu) and (PopupMenu.WindowHandle <> 0) and
        PopupMenu.IsShortCut(Message) then Exit;
      Control := Control.Parent;
    end;
    Form := GetParentForm(Self);
    if (Form <> nil) and Form.IsShortCut(Message) then Exit;
  end;
  with Message do
    if SendAppMessage(CM_APPKEYDOWN, CharCode, KeyData) <> 0 then Exit;
  Result := False;
end;

procedure TCustomDIBControl.SetBounds(aLeft, aTop, aWidth, aHeight: Integer);
var
  X,
  XDelta,
  YDelta            : Integer;
begin
  if FAlreadyMoving then exit;
  if (aLeft=Left) and (aTop=Top) and (aWidth=Width) and (aHeight=Height) then exit;

  try
    FAlreadyMoving := True;
    XDelta := aLeft - Left;
    YDelta := aTop - Top;

    inherited;

    //Move any dependent children
    if (XDelta<>0) or (YDelta<>0) then
      for X:=0 to FChildren.Count-1 do
        if FChildren[X].Control <> nil then with FChildren[X].Control do
          SetBounds(Left + XDelta, Top + YDelta, Width, Height);
  finally
    FAlreadyMoving := False
  end;
end;

procedure TCustomDIBControl.SetParent(AParent: TWinControl);
var
  X                 : Integer;
begin
  if aParent <> nil then begin
    if not (AParent is TCustomDIBContainer) then
      for X:=0 to AParent.ComponentCount-1 do
        if AParent.Components[X] is TCustomDIBContainer then begin
          AParent := TWinControl(AParent.Components[X]);
          Break;
        end;
    if not (AParent is TCustomDIBContainer) then Raise Exception.Create('Parent must be a TDIBContainer');
  end;
  inherited;
end;

procedure TCustomDIBControl.WMPAINT(var Message: TMessage);
var
  SrcX,
  SrcY,
  DstX,
  DstY              : Integer;
begin
  FCanvas.Lock;
  try
    try
      DstX := 0;
      DstY := 0;
      SrcX := Left;
      SrcY := Top;
      if SrcX < 0 then begin
        DstX := Abs(SrcX);
        SrcX := 0;
      end;
      if SrcY < 0 then begin
        DstY := Abs(SrcY);
        SrcY := 0;
      end;
      TCustomDIBContainer(Parent).DIB.Draw(DstX,DstY,Width,Height,ControlDIB,SrcX,SrcY);
      Canvas.handle := FControlDIB.handle;
      Paint;

      if csDesigning in ComponentState then with canvas do begin
        Pen.color := clBlack;
        Pen.Style := psDash;
        Brush.Style := bsClear;
        Rectangle(0,0,Width,Height);
      end;

      with THackAbstractSuperDIB(ControlDIB) do begin
        Opacity := Self.Opacity;
        Draw(Self.Left,Self.Top,Width,Height,TCustomDIBContainer(Parent).DIB,0,0);
      end;
    finally
      FCanvas.Handle := 0;
    end;
  finally
    FCanvas.Unlock;
  end;
end;

function TCustomDIBControl.GetContainer: TCustomDIBContainer;
begin
  Result := TCustomDIBContainer(Parent);
end;

procedure TCustomDIBControl.WndProc(var Message: TMessage);
var
  Handled           : Boolean;
begin
  if (csDesigning in ComponentState) then begin
    inherited;
    exit;
  end;

  if Assigned(FDIBFeatures) then begin
    Handled := False;
    FDIBFeatures.WndProc(Message, Handled);
  end;

  if Message.Msg = WM_LButtonDown then begin
    FLastMouse := Message;
    if MouseRepeat and not FTimer.Enabled then begin
      FTimer.Interval := MouseRepeatInterval;
      FTimer.Enabled := True;
    end;
  end;

  if Message.Msg = WM_MouseMove then begin
    FLastMouse.WParam := Message.WParam;
    FLastMouse.LParam := Message.LParam;
    if FRealMouseInControl and not MouseInControl then perform(CM_MouseEnter,0,0);
  end;

  if Message.Msg = WM_LButtonUp then begin
    FTimer.Enabled := False;
    if FMouseInControl and not FRealMouseInControl then Perform(CM_MouseLeave,0,0);
  end;

  if Message.Msg = CM_MouseEnter then begin
    DoMouseEnter;
    if MouseCapture and MouseRepeat then FTimer.Enabled := True;
    if MouseCapture then exit;
  end;

  if Message.Msg = CM_MouseLeave then begin
    FTimer.Enabled := False;
    DoMouseLeave;
  end;

  if not Handled then inherited;
end;

procedure TCustomDIBControl.SetOpacity(const Value: Byte);
begin
  if Value = FOpacity then exit;
  FOpacity := Value;
  Invalidate;
end;

procedure TCustomDIBControl.Paint;
begin

end;

procedure TCustomDIBControl.RepeatMessage(Sender: TObject);
begin
  if FStoppingRepeat then begin
    FStoppingRepeat := False;
    FTimer.Enabled := False;
  end else
    with FLastMouse do
      Perform(Msg, WParam, lParam);
end;

procedure TCustomDIBControl.Notification(AComponent: TComponent;
  Operation: TOperation);
var
  X                           : Integer;
begin
  if (AComponent = Self) or (csDestroying in ComponentState) then exit;
  inherited;
  if Operation = opRemove then
    for X:=Children.Count-1 downto 0 do
      if Children[X].Control = AComponent then
        FChildren[X].Free;
end;

procedure TCustomDIBControl.DoMouseEnter;
begin
  if not(Focused or MouseInControl) then DoAnyEnter;

  FRealMouseInControl := True;
  FMouseInControl := True;
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TCustomDIBControl.DoMouseLeave;
begin
  FRealMouseInControl := False;
  If MouseCapture then exit;
  FMouseInControl := False;
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);

  if not(Focused or MouseInControl) then DoAnyLeave;
end;

procedure TCustomDIBControl.StopRepeating;
begin
  FStoppingRepeat := True;
end;

procedure TCustomDIBControl.AlterUpdateRect(var R: TRect);
begin

end;

procedure TCustomDIBControl.SetTabOrder(const Value: TTabOrder);
begin
  if csLoading in ComponentState then
    FTabOrder := Value
  else begin
    Container.DIBSetTabOrder(Self, Value);
    FTabOrder := GetTabOrder;
  end;
end;


function TCustomDIBControl.GetTabOrder: TTabOrder;
begin
  Result := Container.DIBGetTabOrder(Self);
end;

procedure TCustomDIBControl.SetFocus;
begin
  Perform(WM_SetFocus,0,0);
end;

procedure TCustomDIBControl.WMKillFocus(var Message: TMessage);
begin
  DoExit;
end;

procedure TCustomDIBControl.WMSetFocus(var Message: TMessage);
begin
  DoEnter;
end;

procedure TCustomDIBControl.DoEnter;
begin
  if not(Focused or MouseInControl) then DoAnyEnter;

  FFocused := True;
  Container.DIBFocusControl(Self);
  Container.SetFocus;
  if Assigned(FOnEnter) then FOnEnter(Self);
end;

procedure TCustomDIBControl.DoExit;
begin
  FFocused := False;
  if Assigned(FOnExit) then FOnExit(Self);
  if not(Focused or MouseInControl) then DoAnyLeave;
end;

procedure TCustomDIBControl.WMKeyDown(var Message: TWMKey);
begin
  if not DoKeyDown(Message) then inherited;
end;

procedure TCustomDIBControl.WMKeyUp(var Message: TWMKey);
begin
  if not DoKeyUp(Message) then inherited;
end;

function TCustomDIBControl.DoKeyDown(var Message: TWMKey): Boolean;
var
  ShiftState: TShiftState;
  Form: TCustomForm;
begin
  Result := True;
  Form := GetParentForm(Self);
  if (Form <> nil) and Form.KeyPreview and
    THackWinControl(Form).DoKeyDown(Message) then Exit;
  with Message do
  begin
    ShiftState := KeyDataToShiftState(KeyData);
    if not (csNoStdEvents in ControlStyle) then
    begin
      KeyDown(CharCode, ShiftState);
      if CharCode = 0 then Exit;
    end;
  end;
  Result := False;
end;

function TCustomDIBControl.DoKeyPress(var Message: TWMKey): Boolean;
var
  Form: TCustomForm;
  Ch: Char;
begin
  Result := True;
  Form := GetParentForm(Self);
  if (Form <> nil) and Form.KeyPreview and
    THackWinControl(Form).DoKeyPress(Message) then Exit;
  if not (csNoStdEvents in ControlStyle) then
    with Message do
    begin
      Ch := Char(CharCode);
      KeyPress(Ch);
      CharCode := Word(Ch);
      if Char(CharCode) = #0 then Exit;
    end;
  Result := False;
end;

function TCustomDIBControl.DoKeyUp(var Message: TWMKey): Boolean;
var
  ShiftState: TShiftState;
  Form: TCustomForm;
begin
  Result := True;
  Form := GetParentForm(Self);
  if (Form <> nil) and Form.KeyPreview and
    THackWinControl(Form).DoKeyUp(Message) then Exit;
  with Message do
  begin
    ShiftState := KeyDataToShiftState(KeyData);
    if not (csNoStdEvents in ControlStyle) then
    begin
      KeyUp(CharCode, ShiftState);
      if CharCode = 0 then Exit;
    end;
  end;
  Result := False;
end;

procedure TCustomDIBControl.WMChar(var Message: TWMKey);
begin
  if not DoKeyPress(Message) then inherited;
end;

procedure TCustomDIBControl.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Assigned(FOnKeyDown) then FOnKeyDown(Self, Key, Shift);
end;

procedure TCustomDIBControl.KeyPress(var Key: Char);
begin
  if Assigned(FOnKeyPress) then FOnKeyPress(Self, Key);
end;

procedure TCustomDIBControl.KeyUp(var Key: Word; Shift: TShiftState);
begin
  if Assigned(FOnKeyUp) then FOnKeyUp(Self, Key, Shift);
end;

procedure TCustomDIBControl.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  with Message do begin
    Result := 0;
    if wkAll in WantedKeys then
      Result := DLGC_WANTALLKEYS
    else begin
      if wkTab in WantedKeys then Result := Result or DLGC_WANTTab;
      if wkArrows in WantedKeys then Result := Result or DLGC_WANTArrows;
    end;
  end;
end;

procedure TCustomDIBControl.CMDialogChar(var Message: TCMDialogChar);
var
  ShiftState                  : TShiftState;
begin
  ShiftState := KeyDataToShiftState(Message.KeyData);

  if (Message.CharCode = Word(FAccelerator)) and
     Enabled and Visible and (ssAlt in ShiftState) and Parent.CanFocus then
  begin
    Click;
    Message.Result := 0;
  end else
    inherited;
end;

procedure TCustomDIBControl.DoAnyEnter;
begin

end;

procedure TCustomDIBControl.DoAnyLeave;
begin

end;

procedure TCustomDIBControl.Click;
begin
  inherited;
  if DIBTabOrder > -1 then SetFocus;
end;

procedure TCustomDIBControl.DoImageChanged(Sender: TObject; Index : Integer;
  Operation: TDIBOperation);
begin
  if not (csDestroying in ComponentState) then
    ImageChanged(Index, Operation);
end;

procedure TCustomDIBControl.ImageChanged(Index: Integer; Operation: TDIBOperation);
begin

end;

function TCustomDIBControl.IsMouseRepeating: Boolean;
begin
  Result := FTimer.Enabled;
end;

procedure TCustomDIBControl.AddIndexProperty(var Index: TDIBImageLink);
begin
  Index := TDIBImageLink.Create(Self);
  FPropertyList.Add(Index);
  Index.OnImageChanged := DoImageChanged;
end;

procedure TCustomDIBControl.SetDIBImageList(const Value: TDIBImageList);
var
  X                           : Integer;
begin
  FDIBImageList := Value;
  for X:=0 to FPropertyList.Count-1 do
    TDIBImageLink(FPropertyList[X]).DIBImageList := Value;
end;


{
The format for a template should be
GUID
NumberOfImages : Integer;
NumberOfProperties : Integer;
-for each image-
  LengthOfDisplayName : Integer;
  DisplayName : PChar;
  LengthOfPropertyName : Integer;
  PropertyName : PChar;
  DIB.SaveDataToStream
-end--
-for each property-
  NewIndexNumber : Integer;
-end--
NumberOfClassProperties : Integer;
--for each property--
  LengthOfPropName : Integer;
  PropName : PChar;
  SkipSize : Integer;  //if this property does not exist
  Data : Binary
--end--
CUSTOM DATA GOES HERE

}

procedure TCustomDIBControl.LoadTemplateFromFile(
  const Filename: TFilename);
var
  FS : TFileStream;
begin
  FS := TFileStream.Create(Filename, fmOpenRead);
  try
    LoadTemplateFromStream(FS);
  finally
    FS.Free;
  end;
end;

procedure TCustomDIBControl.LoadTemplateFromStream(const S: TStream);
var
  I,
  Index,
  DisplayLen,
  PropLen,
  PropertyCount,
  ImageCount : Integer;
  DisplayName,
  PropName : String;

  GUID : TGUID;
  GUIDStr : String;

  FIndexes : TList;
  DIBWrapper : TDIBWrapper;
begin
  if not Assigned(DIBImageList) then
    raise EDIBControlError.Create('No DIB image list has been assigned.');

  FIndexes := TList.Create;
  DIBWrapper := TDIBWrapper.Create(Self);
  try
    S.Read(GUID, SizeOf(TGUID));

    S.Read(ImageCount, SizeOf(Integer));
    S.Read(PropertyCount, SizeOf(Integer));
    GUIDStr := GUIDToString(GUID);

    for I:=0 to ImageCount-1 do begin
      with DIBImageList.DIBImages.AddTemplate(GUIDStr,I) do begin
        DIBWrapper.DIB := DIB;
        S.ReadComponent(DIBWrapper);
        FIndexes.Add(Pointer(Index));
      end;
    end;

    for I:=0 to PropertyCount-1 do begin
      //Get the property name
      S.Read(PropLen, SizeOf(Integer));
      SetLength(PropName, PropLen);
      S.Read(PropName[1], PropLen);

      //Get the display name
      DisplayName := '';
      S.Read(DisplayLen, SizeOf(Integer));
      if DisplayLen > 0 then begin
        SetLength(DisplayName, DisplayLen);
        S.Read(DisplayName[1], DisplayLen);
      end;

      S.Read(Index, SizeOf(Integer));
      if IsPublishedProp(Self, PropName) then begin
        Index := Integer(FIndexes[Index]);
        with TDIBImageLink(GetOrdProp(Self, PropName)) do begin
          DIBIndex := Index;
          if DisplayName <> '' then
            DIBImageList.DIBImages[Index].DisplayName := DisplayName;
        end;
      end;
    end;
    ReadProperties(S);
    Loaded;
  finally
    DIBWrapper.Free;
    FIndexes.Free;
  end;
end;


procedure TCustomDIBControl.SaveTemplateToFile(const Filename: TFilename);
var
  FS : TFileStream;
begin
  FS := TFileStream.Create(Filename, fmCreate);
  try
    SaveTemplateToStream(FS);
  finally
    FS.Free;
  end;
end;

procedure TCustomDIBControl.SaveTemplateToStream(const S: TStream);
var
  I,
  PropertyCount,
  ImageCount,
  DisplayLen,
  PropLen,
  NewIndex,
  Index : Integer;

  DisplayName,
  PropName : String;
  FIndexes : TList;

  GUID : TGUID;

  PropertyStream,
  ImageStream : TMemoryStream;

  DIBWrapper : TDIBWrapper;
begin
  if DIBImageList = nil then exit;

  if CoCreateGUID(GUID) <> S_OK then
    raise EDIBControlError.Create('Could not create a GUID for your new template.');

  FIndexes := TList.Create;
  PropertyStream := TMemoryStream.Create;
  ImageStream := TMemoryStream.Create;
  DIBWrapper := TDIBWrapper.Create(Self);

  try
    PropertyCount := 0;
    ImageCount := 0;

    for I:=0 to FPropertyList.Count-1 do begin
      //Get the name of the property, and the INDEX
      PropName := GetPropertyName(Self, FPropertyList[I]);
      Index := TDIBImageLink(FPropertyList[I]).DIBIndex;

      if (PropName <> '') and (DIBImageList.IsIndexValid(Index)) then begin
        //If it is published, and has a valid pic, we need it
        Inc(PropertyCount);

        //Write length of the property name
        PropLen := Length(PropName);
        PropertyStream.Write(PropLen, SizeOf(Integer));
        //Write the property name
        PropertyStream.Write(PropName[1], PropLen);

        //Write the length of the display name
        DisplayName := TDIBImageLink(FPropertyList[I]).DIBImageList.DIBImages[Index].DisplayName;
        DisplayLen := Length(DisplayName);
        PropertyStream.Write(DisplayLen, SizeOf(Integer));
        if DisplayLen > 0 then
          PropertyStream.Write(DisplayName[1], DisplayLen);

        //See if we have already written this index
        NewIndex := FIndexes.IndexOf(Pointer(Index));
        //If not, we write it
        if NewIndex =-1 then begin
          Inc(ImageCount);
          NewIndex := FIndexes.add(Pointer(Index));
          DIBWrapper.DIB := DIBImageList.DIBImages[Index].DIB;
          ImageStream.WriteComponent(DIBWrapper);
        end;

        //Now write out the remapped index (NOT the original index)
        PropertyStream.Write(NewIndex, SizeOf(Integer));
      end;
    end;

    //Now we have build a property list (with remapped index) and
    //an image list, we stick it all together
    PropertyStream.Seek(0, soFromBeginning);
    ImageStream.Seek(0, soFromBeginning);

    S.Write(GUID, SizeOf(TGUID));

    S.Write(ImageCount, SizeOf(Integer));
    S.Write(PropertyCount, SizeOf(Integer));
    S.CopyFrom(ImageStream, ImageStream.Size);
    S.CopyFrom(PropertyStream, PropertyStream.Size);
    WriteProperties(S);
  finally
    DIBWrapper.Free;
    PropertyStream.Free;
    ImageStream.Free;
    FIndexes.Free;
  end;
end;


procedure TCustomDIBControl.AddTemplateProperty(const Name: String);
begin
  if not IsPublishedProp(Self, Name) then
    raise EDIBControlError.Create('Class ' + ClassName + ' is trying to register ' + Name +
      ' as a template property, and that property does not exist.');
      
  if not (PropType(Self, Name) in
    [tkSet, tkInteger, tkEnumeration, tkInt64, tkChar,tkString, tkWChar, tkLString, tkWString, tkFloat]) then
    raise EDIBControlError.Create('Attempting to template an invalid property type.');

  FPropertyNames.Add(Name);
end;

procedure TCustomDIBControl.ReadProperties(S: TStream);
var
  nProp,
  I                           : Integer;
  FloatVal                    : Extended;
  SkipSize,
  StrSize,
  PropSize,
  OrdVal                      : Longint;
  PropName,
  StrVal                      : String;
begin
  S.Read(nProp, SizeOf(Integer));

  for I:=0 to nProp-1 do begin
    //Read the property name  
    S.Read(PropSize, SizeOf(Integer));
    SetLength(PropName, PropSize);
    S.Read(PropName[1], PropSize);

    //Now the skip size
    S.Read(SkipSize, SizeOf(Longint));

    if not IsPublishedProp(Self, PropName) then
      S.Seek(SkipSize, soFromCurrent)
    else
      //Now the value
      case PropType(Self, PropName) of
        tkSet,
        tkInteger,
        tkEnumeration,
        tkInt64 :
          begin
            S.Read(OrdVal, SizeOf(LongInt));
            SetOrdProp(Self, PropName, OrdVal);
          end;

        tkChar,
        tkString,
        tkWChar,
        tkLString,
        tkWString :
          begin
            S.Read(StrSize, SizeOf(Integer));
            SetLength(StrVal, StrSize);
            if StrSize > 0 then
              S.Read(StrVal[1], StrSize);
            SetStrProp(Self, PropName, StrVal);
          end;

        tkFloat :
          begin
            S.Read(FloatVal, SizeOf(Extended));
            SetFloatProp(Self, PropName, FloatVal);
          end;
      end;
  end;
end;

procedure TCustomDIBControl.WriteProperties(S: TStream);
var
  I                           : Integer;
  FloatVal                    : Extended;
  StrSize,
  PropSize,
  OrdVal                      : Longint;
  PropName,
  StrVal                      : String;
begin
  I := FPropertyNames.Count;
  S.Write(I, SizeOf(Integer));

  for I:=0 to FPropertyNames.Count-1 do begin
    PropName := FPropertyNames[I];
    PropSize := Length(PropName);

    //Write the property name first
    S.Write(PropSize, SizeOf(Integer));
    S.Write(PropName[1], PropSize);

    //Now the value
    case PropType(Self, PropName) of
      tkSet,
      tkInteger,
      tkEnumeration,
      tkInt64 :
        begin
          //Skip size in case this property does not exist
          OrdVal := SizeOf(Longint);
          S.Write(OrdVal, SizeOf(LongInt));

          //The value
          OrdVal := GetOrdProp(Self, PropName);
          S.Write(OrdVal, SizeOf(Longint));
        end;

      tkChar,
      tkString,
      tkWChar,
      tkLString,
      tkWString :
        begin
          //Get the string value
          StrVal := GetStrProp(Self, PropName);
          StrSize := Length(StrVal);

          //Skip size in case this property does not exist
          OrdVal := SizeOf(Longint) + StrSize;
          S.Write(OrdVal, SizeOf(LongInt));

          //Write the string size + data
          S.Write(StrSize, SizeOf(Integer));
          if StrSize > 0 then
            S.Write(StrVal[1], StrSize);
        end;

      tkFloat :
        begin
          //Write the skip size
          OrdVal := SizeOf(Extended);
          S.Write(OrdVal, SizeOf(Longint));

          //Write the property value
          FloatVal := GetFloatProp(Self, PropName);
          S.Write(FloatVal, SizeOf(Extended));
        end;
    end;
  end;
end;

initialization
//  CritMessageSection := TCriticalSection.Create;
//  FChildMessages := TList.Create;
//  AddDIBChildMessage(WM_KeyDown);
//  AddDIBChildMessage(WM_KeyUp);
//  AddDIBChildMessage(WM_Char);
finalization
//  FChildMessages.Free;
//  CritMessageSection.Free;
end.
